@@ -30,82 +30,84 @@
  * containing one token from the string. 
  */
 static void
-exsltStrTokenizeFunction (xmlXPathParserContextPtr ctxt, int nargs) {
+exsltStrTokenizeFunction(xmlXPathParserContextPtr ctxt, int nargs)
+{
+    xsltTransformContextPtr tctxt;
     xmlChar *str, *delimiters, *cur;
     const xmlChar *token, *delimiter;
     xmlNodePtr node;
-    xmlDocPtr doc;
-    xmlXPathObjectPtr ret;
+    xmlDocPtr container;
+    xmlXPathObjectPtr ret = NULL;
 
     if ((nargs < 1) || (nargs > 2)) {
-	xmlXPathSetArityError (ctxt);
-	return;
+        xmlXPathSetArityError(ctxt);
+        return;
     }
 
     if (nargs == 2) {
-	delimiters = xmlXPathPopString (ctxt);
-	if (xmlXPathCheckError(ctxt))
-	    return;
+        delimiters = xmlXPathPopString(ctxt);
+        if (xmlXPathCheckError(ctxt))
+            return;
     } else {
-	delimiters = xmlStrdup((const xmlChar *) "\t\r\n ");
+        delimiters = xmlStrdup((const xmlChar *) "\t\r\n ");
     }
     if (delimiters == NULL)
-	return;
+        return;
 
-    str = xmlXPathPopString (ctxt);
+    str = xmlXPathPopString(ctxt);
     if (xmlXPathCheckError(ctxt) || (str == NULL)) {
-	xmlFree (delimiters);
-	return;
+        xmlFree(delimiters);
+        return;
     }
 
-    doc = xsltXPathGetTransformContext(ctxt)->document->doc;
-    ret = xmlXPathNewNodeSet (NULL);
-    if (ret != NULL) {
-	ret->boolval = 1;
-	/*
-	 * This is a hack: token elements are added as children of a
-	 * fake element node. This is necessary to free them up
-	 * correctly when freeing the node-set.
-	 */
-	ret->user = (void *) xmlNewDocNode (doc, NULL,
-				(const xmlChar *) "fake", NULL);
-	if (ret->user == NULL)
-	    goto error;
+    /* Return a result tree fragment */
+    tctxt = xsltXPathGetTransformContext(ctxt);
+    if (tctxt == NULL) {
+        xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
+	      "exslt:tokenize : internal error tctxt == NULL\n");
+	goto fail;
     }
 
-    for (cur = str, token = str; *cur != 0; cur++) {
-	for (delimiter = delimiters; *delimiter != 0; delimiter++) {
-	    if (*cur == *delimiter) {
-		if (cur == token) {
-		    /* discard empty tokens */
-		    break;
-		}
-		*cur = 0;
-		node = xmlNewDocNode (doc, NULL,
-				      (const xmlChar *) "token", token);
-		*cur = *delimiter;
-		token = cur + 1;
-
-		xmlAddChild ((xmlNodePtr) ret->user, node);
-		xmlXPathNodeSetAdd (ret->nodesetval, node);
-		break;
-	    }
-	}
+    container = xsltCreateRVT(tctxt);
+    if (container != NULL) {
+        xsltRegisterTmpRVT(tctxt, container);
+        ret = xmlXPathNewNodeSet(NULL);
+        if (ret != NULL) {
+            ret->boolval = 0; /* Freeing is not handled there anymore */
+            for (cur = str, token = str; *cur != 0; cur++) {
+                for (delimiter = delimiters; *delimiter != 0; delimiter++) {
+                    if (*cur == *delimiter) {
+                        if (cur == token) {
+                            /* discard empty tokens */
+                            break;
+                        }
+                        *cur = 0;
+                        node = xmlNewChild((xmlNodePtr) container, NULL,
+                                           (const xmlChar *) "token",
+                                           token);
+			xmlXPathNodeSetAddUnique(ret->nodesetval, node);
+                        *cur = *delimiter;
+                        token = cur + 1;
+                        break;
+                    }
+                }
+            }
+            node =
+                xmlNewChild((xmlNodePtr) container, NULL,
+                            (const xmlChar *) "token", token);
+	    xmlXPathNodeSetAddUnique(ret->nodesetval, node);
+        }
     }
-    node = xmlNewDocNode (doc, NULL, (const xmlChar *) "token", token);
-    xmlAddChild ((xmlNodePtr) ret->user, node);
-    xmlXPathNodeSetAdd (ret->nodesetval, node);
 
-    valuePush (ctxt, ret);
-    ret = NULL;		/* hack to prevent freeing ret later */
-
-error:
-    if (ret != NULL)
-	xmlXPathFreeObject (ret);
+fail:
     if (str != NULL)
-	xmlFree(str);
+        xmlFree(str);
     if (delimiters != NULL)
-	xmlFree(delimiters);
+        xmlFree(delimiters);
+    if (ret != NULL)
+        valuePush(ctxt, ret);
+    else
+        valuePush(ctxt, xmlXPathNewNodeSet(NULL));
 }
 
 /**
