@@ -68,53 +68,67 @@ double xmlXPathStringEvalNumber(const xmlChar *str);
 #define IS_BLANK_NODE(n)						\
     (((n)->type == XML_TEXT_NODE) && (xsltIsBlank((n)->content)))
 
-/*
- * Generic function for accessing stacks in the stylesheet
+/**
+ * exclPrefixPush:
+ * @style: the transformation stylesheet
+ * @value:  the excluded prefix to push on the stack
+ *
+ * Push an excluded prefix on the stack
+ *
+ * Returns the new index in the stack or 0 in case of error
  */
-
-#define PUSH_AND_POP(scope, type, name)					\
-scope int name##Push(xsltStylesheetPtr style, type value)	 {	\
-    if (style->name##Max == 0) {					\
-	style->name##Max = 4;						\
-        style->name##Tab = (type *) xmlMalloc(style->name##Max *	\
-	              sizeof(style->name##Tab[0]));			\
-        if (style->name##Tab == NULL) {					\
-	    xmlGenericError(xmlGenericErrorContext,			\
-		    "malloc failed !\n");				\
-	    return(0);							\
-	}								\
-    }									\
-    if (style->name##Nr >= style->name##Max) {				\
-	style->name##Max *= 2;						\
-        style->name##Tab = (type *) xmlRealloc(style->name##Tab,	\
-	             style->name##Max * sizeof(style->name##Tab[0]));	\
-        if (style->name##Tab == NULL) {					\
-	    xmlGenericError(xmlGenericErrorContext,			\
-		    "realloc failed !\n");				\
-	    return(0);							\
-	}								\
-    }									\
-    style->name##Tab[style->name##Nr] = value;				\
-    style->name = value;						\
-    return(style->name##Nr++);						\
-}									\
-scope type name##Pop(xsltStylesheetPtr style)	 {			\
-    type ret;								\
-    if (style->name##Nr <= 0) return(0);				\
-    style->name##Nr--;							\
-    if (style->name##Nr > 0)						\
-	style->name = style->name##Tab[style->name##Nr - 1];		\
-    else								\
-        style->name = NULL;						\
-    ret = style->name##Tab[style->name##Nr];				\
-    style->name##Tab[style->name##Nr] = 0;				\
-    return(ret);							\
-}									\
-
-/*
- * Those macros actually generate the functions
+static int
+exclPrefixPush(xsltStylesheetPtr style, xmlChar * value)
+{
+    if (style->exclPrefixMax == 0) {
+        style->exclPrefixMax = 4;
+        style->exclPrefixTab =
+            (xmlChar * *)xmlMalloc(style->exclPrefixMax *
+                                   sizeof(style->exclPrefixTab[0]));
+        if (style->exclPrefixTab == NULL) {
+            xmlGenericError(xmlGenericErrorContext, "malloc failed !\n");
+            return (0);
+        }
+    }
+    if (style->exclPrefixNr >= style->exclPrefixMax) {
+        style->exclPrefixMax *= 2;
+        style->exclPrefixTab =
+            (xmlChar * *)xmlRealloc(style->exclPrefixTab,
+                                    style->exclPrefixMax *
+                                    sizeof(style->exclPrefixTab[0]));
+        if (style->exclPrefixTab == NULL) {
+            xmlGenericError(xmlGenericErrorContext, "realloc failed !\n");
+            return (0);
+        }
+    }
+    style->exclPrefixTab[style->exclPrefixNr] = value;
+    style->exclPrefix = value;
+    return (style->exclPrefixNr++);
+}
+/**
+ * exclPrefixPop:
+ * @style: the transformation stylesheet
+ *
+ * Pop an excluded prefix value from the stack
+ *
+ * Returns the stored excluded prefix value
  */
-PUSH_AND_POP(static, xmlChar *, exclPrefix)
+static xmlChar *
+exclPrefixPop(xsltStylesheetPtr style)
+{
+    xmlChar *ret;
+
+    if (style->exclPrefixNr <= 0)
+        return (0);
+    style->exclPrefixNr--;
+    if (style->exclPrefixNr > 0)
+        style->exclPrefix = style->exclPrefixTab[style->exclPrefixNr - 1];
+    else
+        style->exclPrefix = NULL;
+    ret = style->exclPrefixTab[style->exclPrefixNr];
+    style->exclPrefixTab[style->exclPrefixNr] = 0;
+    return (ret);
+}
 
 /************************************************************************
  *									*
@@ -240,20 +254,25 @@ xsltFreeDecimalFormatList(xsltStylesheetPtr self)
  * @name: the decimal-format name to find
  *
  * Find decimal-format by name
+ *
+ * Returns the xsltDecimalFormatPtr
  */
 xsltDecimalFormatPtr
 xsltDecimalFormatGetByName(xsltStylesheetPtr sheet, xmlChar *name)
 {
-    xsltDecimalFormatPtr result;
+    xsltDecimalFormatPtr result = NULL;
 
     if (name == NULL)
 	return sheet->decimalFormat;
-    
-    for (result = sheet->decimalFormat->next;
-	 result != NULL;
-	 result = result->next) {
-	if (xmlStrEqual(name, result->name))
-	    break; /* for */
+
+    while (sheet != NULL) {
+	for (result = sheet->decimalFormat->next;
+	     result != NULL;
+	     result = result->next) {
+	    if (xmlStrEqual(name, result->name))
+		return result;
+	}
+	sheet = xsltNextImport(sheet);
     }
     return result;
 }
@@ -534,6 +553,9 @@ xsltGetInheritedNsList(xsltStylesheetPtr style,
             while (cur != NULL) {
 		if (xmlStrEqual(cur->href, XSLT_NAMESPACE))
 		    goto skip_ns;
+		if ((cur->prefix != NULL) &&
+		    (xsltCheckExtPrefix(style, cur->prefix)))
+		    goto skip_ns;
 		for (i = 0;i < style->exclPrefixNr;i++) {
 		    if (xmlStrEqual(cur->href, style->exclPrefixTab[i]))
 			goto skip_ns;
@@ -636,7 +658,7 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
 
 	URI = xsltGetQNameURI(cur, &prop);
 	if (prop == NULL) {
-	    style->errors++;
+	    if (style != NULL) style->errors++;
 	} else if (URI == NULL) {
             if ((xmlStrEqual(prop, (const xmlChar *) "xml")) ||
                 (xmlStrEqual(prop, (const xmlChar *) "html")) ||
@@ -645,7 +667,7 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
             } else {
 		xsltTransformError(NULL, style, cur,
                                  "invalid value for method: %s\n", prop);
-                style->warnings++;
+                if (style != NULL) style->warnings++;
             }
 	} else {
 	    style->method = prop;
@@ -681,7 +703,7 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
         } else {
 	    xsltTransformError(NULL, style, cur,
                              "invalid value for standalone: %s\n", prop);
-            style->warnings++;
+            if (style != NULL) style->warnings++;
         }
         xmlFree(prop);
     }
@@ -695,7 +717,7 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
         } else {
 	    xsltTransformError(NULL, style, cur,
                              "invalid value for indent: %s\n", prop);
-            style->warnings++;
+            if (style != NULL) style->warnings++;
         }
         xmlFree(prop);
     }
@@ -711,7 +733,7 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
 	    xsltTransformError(NULL, style, cur,
                              "invalid value for omit-xml-declaration: %s\n",
                              prop);
-            style->warnings++;
+            if (style != NULL) style->warnings++;
         }
         xmlFree(prop);
     }
@@ -745,7 +767,7 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
 
 		URI = xsltGetQNameURI(cur, &element);
 		if (element == NULL) {
-		    style->errors++;
+		    if (style != NULL) style->errors++;
 		} else {
 		    xmlHashAddEntry2(style->cdataSection, element, URI,
 			             (void *) "cdata");
@@ -791,14 +813,14 @@ xsltParseStylesheetDecimalFormat(xsltStylesheetPtr style, xmlNodePtr cur)
 	if (format != NULL) {
 	    xsltTransformError(NULL, style, cur,
 	 "xsltParseStylestyleDecimalFormat: %s already exists\n", prop);
-	    style->warnings++;
+	    if (style != NULL) style->warnings++;
 	    return;
 	}
 	format = xsltNewDecimalFormat(prop);
 	if (format == NULL) {
 	    xsltTransformError(NULL, style, cur,
      "xsltParseStylestyleDecimalFormat: failed creating new decimal-format\n");
-	    style->errors++;
+	    if (style != NULL) style->errors++;
 	    return;
 	}
 	/* Append new decimal-format structure */
@@ -893,7 +915,7 @@ xsltParseStylesheetPreserveSpace(xsltStylesheetPtr style, xmlNodePtr cur) {
     if (elements == NULL) {
 	xsltTransformError(NULL, style, cur,
 	    "xsltParseStylesheetPreserveSpace: missing elements attribute\n");
-	style->warnings++;
+	if (style != NULL) style->warnings++;
 	return;
     }
 
@@ -974,7 +996,7 @@ xsltParseStylesheetExtPrefix(xsltStylesheetPtr style, xmlNodePtr cur) {
 		xsltTransformError(NULL, style, cur,
 	    "xsl:extension-element-prefix : undefined namespace %s\n",
 	                         prefix);
-		style->warnings++;
+		if (style != NULL) style->warnings++;
 	    } else {
 #ifdef WITH_XSLT_DEBUG_PARSING
 		xsltGenericDebug(xsltGenericDebugContext,
@@ -1010,7 +1032,7 @@ xsltParseStylesheetStripSpace(xsltStylesheetPtr style, xmlNodePtr cur) {
     if (elements == NULL) {
 	xsltTransformError(NULL, style, cur,
 	    "xsltParseStylesheetStripSpace: missing elements attribute\n");
-	style->warnings++;
+	if (style != NULL) style->warnings++;
 	return;
     }
 
@@ -1094,7 +1116,7 @@ xsltParseStylesheetExcludePrefix(xsltStylesheetPtr style, xmlNodePtr cur) {
 		xsltTransformError(NULL, style, cur,
 	    "xsl:exclude-result-prefixes : undefined namespace %s\n",
 	                         prefix);
-		style->warnings++;
+		if (style != NULL) style->warnings++;
 	    } else {
 #ifdef WITH_XSLT_DEBUG_PARSING
 		xsltGenericDebug(xsltGenericDebugContext,
@@ -1286,7 +1308,7 @@ xsltGatherNamespaces(xsltStylesheetPtr style) {
 			if (style->nsHash == NULL) {
 			    xsltTransformError(NULL, style, cur,
 		 "xsltGatherNamespaces: failed to create hash table\n");
-			    style->errors++;
+			    if (style != NULL) style->errors++;
 			    return;
 			}
 		    }
@@ -1294,7 +1316,7 @@ xsltGatherNamespaces(xsltStylesheetPtr style) {
 		    if ((URI != NULL) && (!xmlStrEqual(URI, ns->href))) {
 			xsltTransformError(NULL, style, cur,
 	     "Namespaces prefix %s used for multiple namespaces\n");
-			style->warnings++;
+			if (style != NULL) style->warnings++;
 		    } else if (URI == NULL) {
 			xmlHashUpdateEntry(style->nsHash, ns->prefix,
 			    (void *) ns->href, (xmlHashDeallocator)xmlFree);
@@ -1390,18 +1412,22 @@ xsltParseTemplateContent(xsltStylesheetPtr style, xmlNodePtr templ) {
 						(const xmlChar *)"no")){
 			    xsltTransformError(NULL, style, cur,
 	     "xsl:text: disable-output-escaping allows only yes or no\n");
-			    style->warnings++;
+			    if (style != NULL) style->warnings++;
 
 			}
 			xmlFree(prop);
 		    }
 
 		    while (text != NULL) {
+			if (text->type == XML_COMMENT_NODE) {
+			    text = text->next;
+			    continue;
+			}
 			if ((text->type != XML_TEXT_NODE) &&
 			     (text->type != XML_CDATA_SECTION_NODE)) {
 			    xsltTransformError(NULL, style, cur,
 		 "xsltParseTemplateContent: xslt:text content problem\n");
-			    style->errors++;
+			    if (style != NULL) style->errors++;
 			    break;
 			}
 			if (noesc)
@@ -1493,7 +1519,7 @@ skip_children:
             cur = cur->next;
 	    xsltTransformError(NULL, style, cur,
 		"xsltParseTemplateContent: ignoring misplaced param element\n");
-	    style->warnings++;
+	    if (style != NULL) style->warnings++;
 	    xmlUnlinkNode(param);
 	    xmlFreeNode(param);
 	    continue;
@@ -1531,7 +1557,7 @@ xsltParseStylesheetKey(xsltStylesheetPtr style, xmlNodePtr key) {
 
 	URI = xsltGetQNameURI(key, &prop);
 	if (prop == NULL) {
-	    style->errors++;
+	    if (style != NULL) style->errors++;
 	    goto error;
 	} else {
 	    name = prop;
@@ -1545,7 +1571,7 @@ xsltParseStylesheetKey(xsltStylesheetPtr style, xmlNodePtr key) {
     } else {
 	xsltTransformError(NULL, style, key,
 	    "xsl:key : error missing name\n");
-	style->errors++;
+	if (style != NULL) style->errors++;
 	goto error;
     }
 
@@ -1553,7 +1579,7 @@ xsltParseStylesheetKey(xsltStylesheetPtr style, xmlNodePtr key) {
     if (match == NULL) {
 	xsltTransformError(NULL, style, key,
 	    "xsl:key : error missing match\n");
-	style->errors++;
+	if (style != NULL) style->errors++;
 	goto error;
     }
 
@@ -1561,7 +1587,7 @@ xsltParseStylesheetKey(xsltStylesheetPtr style, xmlNodePtr key) {
     if (use == NULL) {
 	xsltTransformError(NULL, style, key,
 	    "xsl:key : error missing use\n");
-	style->errors++;
+	if (style != NULL) style->errors++;
 	goto error;
     }
 
@@ -1631,7 +1657,7 @@ xsltParseStylesheetTemplate(xsltStylesheetPtr style, xmlNodePtr template) {
 
 	URI = xsltGetQNameURI(template, &prop);
 	if (prop == NULL) {
-	    style->errors++;
+	    if (style != NULL) style->errors++;
 	    goto error;
 	} else {
 	    mode = prop;
@@ -1672,7 +1698,7 @@ xsltParseStylesheetTemplate(xsltStylesheetPtr style, xmlNodePtr template) {
 
 	URI = xsltGetQNameURI(template, &prop);
 	if (prop == NULL) {
-	    style->errors++;
+	    if (style != NULL) style->errors++;
 	    goto error;
 	} else {
 	    ret->name = prop;
@@ -1719,14 +1745,14 @@ xsltParseStylesheetTop(xsltStylesheetPtr style, xmlNodePtr top) {
     if (prop == NULL) {
 	xsltTransformError(NULL, style, top,
 	    "xsl:version is missing: document may not be a stylesheet\n");
-	style->warnings++;
+	if (style != NULL) style->warnings++;
     } else {
 	if ((!xmlStrEqual(prop, (const xmlChar *)"1.0")) &&
             (!xmlStrEqual(prop, (const xmlChar *)"1.1"))) {
 	    xsltTransformError(NULL, style, top,
 		"xsl:version: only 1.0 features are supported\n");
 	     /* TODO set up compatibility when not XSLT 1.0 */
-	    style->warnings++;
+	    if (style != NULL) style->warnings++;
 	}
 	xmlFree(prop);
     }
@@ -1745,7 +1771,7 @@ xsltParseStylesheetTop(xsltStylesheetPtr style, xmlNodePtr top) {
 	    }
 	    if (IS_XSLT_ELEM(cur) && IS_XSLT_NAME(cur, "import")) {
 		    if (xsltParseStylesheetImport(style, cur) != 0)
-			    style->errors++;
+			    if (style != NULL) style->errors++;
 	    } else
 		    break;
 	    cur = cur->next;
@@ -1764,7 +1790,7 @@ xsltParseStylesheetTop(xsltStylesheetPtr style, xmlNodePtr top) {
 		xsltTransformError(NULL, style, cur,
 		    "misplaced text element: '%s'\n", cur->content);
 	    }
-	    style->errors++;
+	    if (style != NULL) style->errors++;
             cur = cur->next;
 	    continue;
 	}
@@ -1772,7 +1798,7 @@ xsltParseStylesheetTop(xsltStylesheetPtr style, xmlNodePtr top) {
 	    xsltGenericError(xsltGenericErrorContext,
 		     "Found a top-level element %s with null namespace URI\n",
 		     cur->name);
-	    style->errors++;
+	    if (style != NULL) style->errors++;
 	    cur = cur->next;
 	    continue;
 	}
@@ -1795,10 +1821,10 @@ xsltParseStylesheetTop(xsltStylesheetPtr style, xmlNodePtr top) {
 	if (IS_XSLT_NAME(cur, "import")) {
 	    xsltTransformError(NULL, style, cur,
 			"xsltParseStylesheetTop: ignoring misplaced import element\n");
-	    style->errors++;
+	    if (style != NULL) style->errors++;
     } else if (IS_XSLT_NAME(cur, "include")) {
 	    if (xsltParseStylesheetInclude(style, cur) != 0)
-			style->errors++;
+		if (style != NULL) style->errors++;
     } else if (IS_XSLT_NAME(cur, "strip-space")) {
 	    xsltParseStylesheetStripSpace(style, cur);
     } else if (IS_XSLT_NAME(cur, "preserve-space")) {
@@ -1826,7 +1852,7 @@ xsltParseStylesheetTop(xsltStylesheetPtr style, xmlNodePtr top) {
 	    xsltTransformError(NULL, style, cur,
 			"xsltParseStylesheetTop: ignoring unknown %s element\n",
 			cur->name);
-	    style->warnings++;
+	    if (style != NULL) style->warnings++;
 	}
 	cur = cur->next;
     }
@@ -1925,7 +1951,38 @@ xsltParseStylesheetProcess(xsltStylesheetPtr ret, xmlDocPtr doc) {
 	template->content = doc->children;
 	xsltAddTemplate(ret, template, NULL, NULL);
     }
-    xsltResolveStylesheetAttributeSet(ret);
+
+    return(ret);
+}
+
+/**
+ * xsltParseStylesheetImportedDoc:
+ * @doc:  and xmlDoc parsed XML
+ *
+ * parse an XSLT stylesheet building the associated structures
+ * except the processing not needed for imported documents.
+ *
+ * Returns a new XSLT stylesheet structure.
+ */
+
+xsltStylesheetPtr
+xsltParseStylesheetImportedDoc(xmlDocPtr doc) {
+    xsltStylesheetPtr ret;
+
+    if (doc == NULL)
+	return(NULL);
+
+    ret = xsltNewStylesheet();
+    if (ret == NULL)
+	return(NULL);
+    
+    ret->doc = doc;
+    xsltGatherNamespaces(ret);
+    if (xsltParseStylesheetProcess(ret, doc) == NULL) {
+	ret->doc = NULL;
+	xsltFreeStylesheet(ret);
+	ret = NULL;
+    }
 
     return(ret);
 }
@@ -1943,20 +2000,11 @@ xsltStylesheetPtr
 xsltParseStylesheetDoc(xmlDocPtr doc) {
     xsltStylesheetPtr ret;
 
-    if (doc == NULL)
-	return(NULL);
-
-    ret = xsltNewStylesheet();
+    ret = xsltParseStylesheetImportedDoc(doc);
     if (ret == NULL)
 	return(NULL);
-    
-    ret->doc = doc;
-    xsltGatherNamespaces(ret);
-	if (xsltParseStylesheetProcess(ret, doc) == NULL) {
-		ret->doc = NULL;
-		xsltFreeStylesheet(ret);
-		ret = NULL;
-	}
+
+    xsltResolveStylesheetAttributeSet(ret);
 
     return(ret);
 }
