@@ -1,6 +1,5 @@
 /* Generated */
 
-#include "config.h"
 #include <libxslt/xsltconfig.h>
 #include "libxslt_wrap.h"
 #include "libxslt-py.h"
@@ -21,6 +20,34 @@ libxslt_xsltTransformGetInstruction(ATTRIBUTE_UNUSED PyObject *self, PyObject *a
     return(py_retval);
 }
 
+PyObject *
+libxslt_xsltParseStylesheetImportedDoc(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
+    PyObject *py_retval;
+    xsltStylesheetPtr c_retval;
+    xmlDocPtr doc;
+    PyObject *pyobj_doc;
+
+    if (!PyArg_ParseTuple(args, (char *)"O:xsltParseStylesheetImportedDoc", &pyobj_doc))
+        return(NULL);
+    doc = (xmlDocPtr) PyxmlNode_Get(pyobj_doc);
+
+    c_retval = xsltParseStylesheetImportedDoc(doc);
+    py_retval = libxslt_xsltStylesheetPtrWrap((xsltStylesheetPtr) c_retval);
+    return(py_retval);
+}
+
+PyObject *
+libxslt_xsltCalibrateAdjust(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
+    long delta;
+
+    if (!PyArg_ParseTuple(args, (char *)"i:xsltCalibrateAdjust", &delta))
+        return(NULL);
+
+    xsltCalibrateAdjust(delta);
+    Py_INCREF(Py_None);
+    return(Py_None);
+}
+
 PyObject *
 libxslt_xsltGenerateIdFunction(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
     xmlXPathParserContextPtr ctxt;
@@ -112,6 +139,22 @@ libxslt_xsltMessage(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
     return(Py_None);
 }
 
+PyObject *
+libxslt_xsltCreateRVT(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
+    PyObject *py_retval;
+    xmlDocPtr c_retval;
+    xsltTransformContextPtr ctxt;
+    PyObject *pyobj_ctxt;
+
+    if (!PyArg_ParseTuple(args, (char *)"O:xsltCreateRVT", &pyobj_ctxt))
+        return(NULL);
+    ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
+
+    c_retval = xsltCreateRVT(ctxt);
+    py_retval = libxml_xmlDocPtrWrap((xmlDocPtr) c_retval);
+    return(py_retval);
+}
+
 PyObject *
 libxslt_xsltParseStylesheetProcess(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
     PyObject *py_retval;
@@ -194,18 +237,35 @@ libxslt_xsltStylesheetGetImports(ATTRIBUTE_UNUSED PyObject *self, PyObject *args
     return(py_retval);
 }
 
+PyObject *
+libxslt_xsltAttrTemplateValueProcess(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
+    PyObject *py_retval;
+    xmlChar * c_retval;
+    xsltTransformContextPtr ctxt;
+    PyObject *pyobj_ctxt;
+    xmlChar * str;
+
+    if (!PyArg_ParseTuple(args, (char *)"Oz:xsltAttrTemplateValueProcess", &pyobj_ctxt, &str))
+        return(NULL);
+    ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
+
+    c_retval = xsltAttrTemplateValueProcess(ctxt, str);
+    py_retval = libxml_xmlCharPtrWrap((xmlChar *) c_retval);
+    return(py_retval);
+}
+
 PyObject *
 libxslt_xsltNextImport(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
     PyObject *py_retval;
     xsltStylesheetPtr c_retval;
-    xsltStylesheetPtr style;
-    PyObject *pyobj_style;
+    xsltStylesheetPtr cur;
+    PyObject *pyobj_cur;
 
-    if (!PyArg_ParseTuple(args, (char *)"O:xsltNextImport", &pyobj_style))
+    if (!PyArg_ParseTuple(args, (char *)"O:xsltNextImport", &pyobj_cur))
         return(NULL);
-    style = (xsltStylesheetPtr) Pystylesheet_Get(pyobj_style);
+    cur = (xsltStylesheetPtr) Pystylesheet_Get(pyobj_cur);
 
-    c_retval = xsltNextImport(style);
+    c_retval = xsltNextImport(cur);
     py_retval = libxslt_xsltStylesheetPtrWrap((xsltStylesheetPtr) c_retval);
     return(py_retval);
 }
@@ -356,6 +416,20 @@ libxslt_xsltStylePreCompute(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
     return(Py_None);
 }
 
+PyObject *
+libxslt_xsltParseStylesheetFile(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
+    PyObject *py_retval;
+    xsltStylesheetPtr c_retval;
+    xmlChar * filename;
+
+    if (!PyArg_ParseTuple(args, (char *)"z:xsltParseStylesheetFile", &filename))
+        return(NULL);
+
+    c_retval = xsltParseStylesheetFile(filename);
+    py_retval = libxslt_xsltStylesheetPtrWrap((xsltStylesheetPtr) c_retval);
+    return(py_retval);
+}
+
 PyObject *
 libxslt_xsltCopyNamespaceList(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
     PyObject *py_retval;
@@ -584,18 +658,6 @@ libxslt_xsltCopyNamespace(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
     return(py_retval);
 }
 
-PyObject *
-libxslt_xsltCalibrateAdjust(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
-    long delta;
-
-    if (!PyArg_ParseTuple(args, (char *)"i:xsltCalibrateAdjust", &delta))
-        return(NULL);
-
-    xsltCalibrateAdjust(delta);
-    Py_INCREF(Py_None);
-    return(Py_None);
-}
-
 PyObject *
 libxslt_xsltSaveProfiling(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
     xsltTransformContextPtr ctxt;
@@ -659,35 +721,31 @@ libxslt_xsltCleanupTemplates(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
 }
 
 PyObject *
-libxslt_xsltCheckExtPrefix(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
-    PyObject *py_retval;
-    int c_retval;
-    xsltStylesheetPtr style;
-    PyObject *pyobj_style;
-    xmlChar * prefix;
+libxslt_xsltDocumentFunction(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
+    xmlXPathParserContextPtr ctxt;
+    PyObject *pyobj_ctxt;
+    int nargs;
 
-    if (!PyArg_ParseTuple(args, (char *)"Oz:xsltCheckExtPrefix", &pyobj_style, &prefix))
+    if (!PyArg_ParseTuple(args, (char *)"Oi:xsltDocumentFunction", &pyobj_ctxt, &nargs))
         return(NULL);
-    style = (xsltStylesheetPtr) Pystylesheet_Get(pyobj_style);
+    ctxt = (xmlXPathParserContextPtr) PyxmlXPathParserContext_Get(pyobj_ctxt);
 
-    c_retval = xsltCheckExtPrefix(style, prefix);
-    py_retval = libxml_intWrap((int) c_retval);
-    return(py_retval);
+    xsltDocumentFunction(ctxt, nargs);
+    Py_INCREF(Py_None);
+    return(Py_None);
 }
 
 PyObject *
-libxslt_xsltParseStylesheetOutput(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
-    xsltStylesheetPtr style;
-    PyObject *pyobj_style;
-    xmlNodePtr cur;
-    PyObject *pyobj_cur;
+libxslt_xsltElementAvailableFunction(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
+    xmlXPathParserContextPtr ctxt;
+    PyObject *pyobj_ctxt;
+    int nargs;
 
-    if (!PyArg_ParseTuple(args, (char *)"OO:xsltParseStylesheetOutput", &pyobj_style, &pyobj_cur))
+    if (!PyArg_ParseTuple(args, (char *)"Oi:xsltElementAvailableFunction", &pyobj_ctxt, &nargs))
         return(NULL);
-    style = (xsltStylesheetPtr) Pystylesheet_Get(pyobj_style);
-    cur = (xmlNodePtr) PyxmlNode_Get(pyobj_cur);
+    ctxt = (xmlXPathParserContextPtr) PyxmlXPathParserContext_Get(pyobj_ctxt);
 
-    xsltParseStylesheetOutput(style, cur);
+    xsltElementAvailableFunction(ctxt, nargs);
     Py_INCREF(Py_None);
     return(Py_None);
 }
@@ -854,6 +912,23 @@ libxslt_xsltNewTransformContext(ATTRIBUTE_UNUSED PyObject *self, PyObject *args)
     return(py_retval);
 }
 
+PyObject *
+libxslt_xsltCheckExtPrefix(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
+    PyObject *py_retval;
+    int c_retval;
+    xsltStylesheetPtr style;
+    PyObject *pyobj_style;
+    xmlChar * prefix;
+
+    if (!PyArg_ParseTuple(args, (char *)"Oz:xsltCheckExtPrefix", &pyobj_style, &prefix))
+        return(NULL);
+    style = (xsltStylesheetPtr) Pystylesheet_Get(pyobj_style);
+
+    c_retval = xsltCheckExtPrefix(style, prefix);
+    py_retval = libxml_intWrap((int) c_retval);
+    return(py_retval);
+}
+
 PyObject *
 libxslt_xsltRegisterAllFunctions(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
     xmlXPathContextPtr ctxt;
@@ -868,6 +943,26 @@ libxslt_xsltRegisterAllFunctions(ATTRIBUTE_UNUSED PyObject *self, PyObject *args
     return(Py_None);
 }
 
+PyObject *
+libxslt_xsltAttrTemplateValueProcessNode(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
+    PyObject *py_retval;
+    xmlChar * c_retval;
+    xsltTransformContextPtr ctxt;
+    PyObject *pyobj_ctxt;
+    xmlChar * str;
+    xmlNodePtr node;
+    PyObject *pyobj_node;
+
+    if (!PyArg_ParseTuple(args, (char *)"OzO:xsltAttrTemplateValueProcessNode", &pyobj_ctxt, &str, &pyobj_node))
+        return(NULL);
+    ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
+    node = (xmlNodePtr) PyxmlNode_Get(pyobj_node);
+
+    c_retval = xsltAttrTemplateValueProcessNode(ctxt, str, node);
+    py_retval = libxml_xmlCharPtrWrap((xmlChar *) c_retval);
+    return(py_retval);
+}
+
 PyObject *
 libxslt_xsltFreeTemplateHashes(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
     xsltStylesheetPtr style;
@@ -882,6 +977,25 @@ libxslt_xsltFreeTemplateHashes(ATTRIBUTE_UNUSED PyObject *self, PyObject *args)
     return(Py_None);
 }
 
+PyObject *
+libxslt_xsltRegisterTmpRVT(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
+    PyObject *py_retval;
+    int c_retval;
+    xsltTransformContextPtr ctxt;
+    PyObject *pyobj_ctxt;
+    xmlDocPtr RVT;
+    PyObject *pyobj_RVT;
+
+    if (!PyArg_ParseTuple(args, (char *)"OO:xsltRegisterTmpRVT", &pyobj_ctxt, &pyobj_RVT))
+        return(NULL);
+    ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
+    RVT = (xmlDocPtr) PyxmlNode_Get(pyobj_RVT);
+
+    c_retval = xsltRegisterTmpRVT(ctxt, RVT);
+    py_retval = libxml_intWrap((int) c_retval);
+    return(py_retval);
+}
+
 PyObject *
 libxslt_xsltUnregisterExtModuleTopLevel(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
     PyObject *py_retval;
@@ -964,6 +1078,20 @@ libxslt_xsltTransformGetContext(ATTRIBUTE_UNUSED PyObject *self, PyObject *args)
     return(py_retval);
 }
 
+PyObject *
+libxslt_xsltFreeRVTs(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
+    xsltTransformContextPtr ctxt;
+    PyObject *pyobj_ctxt;
+
+    if (!PyArg_ParseTuple(args, (char *)"O:xsltFreeRVTs", &pyobj_ctxt))
+        return(NULL);
+    ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
+
+    xsltFreeRVTs(ctxt);
+    Py_INCREF(Py_None);
+    return(Py_None);
+}
+
 PyObject *
 libxslt_xsltRegisterTestModule(ATTRIBUTE_UNUSED PyObject *self,ATTRIBUTE_UNUSED  PyObject *args) {
 
@@ -1045,16 +1173,16 @@ libxslt_xsltAttrTemplateProcess(ATTRIBUTE_UNUSED PyObject *self, PyObject *args)
     PyObject *pyobj_ctxt;
     xmlNodePtr target;
     PyObject *pyobj_target;
-    xmlAttrPtr attr;
-    PyObject *pyobj_attr;
+    xmlAttrPtr cur;
+    PyObject *pyobj_cur;
 
-    if (!PyArg_ParseTuple(args, (char *)"OOO:xsltAttrTemplateProcess", &pyobj_ctxt, &pyobj_target, &pyobj_attr))
+    if (!PyArg_ParseTuple(args, (char *)"OOO:xsltAttrTemplateProcess", &pyobj_ctxt, &pyobj_target, &pyobj_cur))
         return(NULL);
     ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
     target = (xmlNodePtr) PyxmlNode_Get(pyobj_target);
-    attr = (xmlAttrPtr) PyxmlNode_Get(pyobj_attr);
+    cur = (xmlAttrPtr) PyxmlNode_Get(pyobj_cur);
 
-    c_retval = xsltAttrTemplateProcess(ctxt, target, attr);
+    c_retval = xsltAttrTemplateProcess(ctxt, target, cur);
     py_retval = libxml_xmlNodePtrWrap((xmlNodePtr) c_retval);
     return(py_retval);
 }
@@ -1121,37 +1249,6 @@ libxslt_xsltStylesheetGetEncoding(ATTRIBUTE_UNUSED PyObject *self, PyObject *arg
     return(py_retval);
 }
 
-PyObject *
-libxslt_xsltElementAvailableFunction(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
-    xmlXPathParserContextPtr ctxt;
-    PyObject *pyobj_ctxt;
-    int nargs;
-
-    if (!PyArg_ParseTuple(args, (char *)"Oi:xsltElementAvailableFunction", &pyobj_ctxt, &nargs))
-        return(NULL);
-    ctxt = (xmlXPathParserContextPtr) PyxmlXPathParserContext_Get(pyobj_ctxt);
-
-    xsltElementAvailableFunction(ctxt, nargs);
-    Py_INCREF(Py_None);
-    return(Py_None);
-}
-
-PyObject *
-libxslt_xsltTransformGetStyle(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
-    PyObject *py_retval;
-    xsltStylesheetPtr c_retval;
-    xsltTransformContextPtr ctxt;
-    PyObject *pyobj_ctxt;
-
-    if (!PyArg_ParseTuple(args, (char *)"O:xsltTransformGetStyle", &pyobj_ctxt))
-        return(NULL);
-    ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
-
-    c_retval = ctxt->style;
-    py_retval = libxslt_xsltStylesheetPtrWrap((xsltStylesheetPtr) c_retval);
-    return(py_retval);
-}
-
 PyObject *
 libxslt_xsltTransformGetMode(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
     PyObject *py_retval;
@@ -1213,6 +1310,23 @@ libxslt_xsltParseStylesheetParam(ATTRIBUTE_UNUSED PyObject *self, PyObject *args
     return(Py_None);
 }
 
+PyObject *
+libxslt_xsltParseStylesheetOutput(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
+    xsltStylesheetPtr style;
+    PyObject *pyobj_style;
+    xmlNodePtr cur;
+    PyObject *pyobj_cur;
+
+    if (!PyArg_ParseTuple(args, (char *)"OO:xsltParseStylesheetOutput", &pyobj_style, &pyobj_cur))
+        return(NULL);
+    style = (xsltStylesheetPtr) Pystylesheet_Get(pyobj_style);
+    cur = (xmlNodePtr) PyxmlNode_Get(pyobj_cur);
+
+    xsltParseStylesheetOutput(style, cur);
+    Py_INCREF(Py_None);
+    return(Py_None);
+}
+
 PyObject *
 libxslt_xsltFreeStylePreComps(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
     xsltStylesheetPtr style;
@@ -1341,19 +1455,19 @@ PyObject *
 libxslt_xsltSaveResultToFilename(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
     PyObject *py_retval;
     int c_retval;
-    char * URI;
+    char * URL;
     xmlDocPtr result;
     PyObject *pyobj_result;
     xsltStylesheetPtr style;
     PyObject *pyobj_style;
     int compression;
 
-    if (!PyArg_ParseTuple(args, (char *)"zOOi:xsltSaveResultToFilename", &URI, &pyobj_result, &pyobj_style, &compression))
+    if (!PyArg_ParseTuple(args, (char *)"zOOi:xsltSaveResultToFilename", &URL, &pyobj_result, &pyobj_style, &compression))
         return(NULL);
     result = (xmlDocPtr) PyxmlNode_Get(pyobj_result);
     style = (xsltStylesheetPtr) Pystylesheet_Get(pyobj_style);
 
-    c_retval = xsltSaveResultToFilename(URI, result, style, compression);
+    c_retval = xsltSaveResultToFilename(URL, result, style, compression);
     py_retval = libxml_intWrap((int) c_retval);
     return(py_retval);
 }
@@ -1373,6 +1487,22 @@ libxslt_xsltSystemPropertyFunction(ATTRIBUTE_UNUSED PyObject *self, PyObject *ar
     return(Py_None);
 }
 
+PyObject *
+libxslt_xsltGetProfileInformation(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
+    PyObject *py_retval;
+    xmlDocPtr c_retval;
+    xsltTransformContextPtr ctxt;
+    PyObject *pyobj_ctxt;
+
+    if (!PyArg_ParseTuple(args, (char *)"O:xsltGetProfileInformation", &pyobj_ctxt))
+        return(NULL);
+    ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
+
+    c_retval = xsltGetProfileInformation(ctxt);
+    py_retval = libxml_xmlDocPtrWrap((xmlDocPtr) c_retval);
+    return(py_retval);
+}
+
 PyObject *
 libxslt_xsltInitCtxtExts(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
     PyObject *py_retval;
@@ -1389,6 +1519,25 @@ libxslt_xsltInitCtxtExts(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
     return(py_retval);
 }
 
+PyObject *
+libxslt_xsltRegisterPersistRVT(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
+    PyObject *py_retval;
+    int c_retval;
+    xsltTransformContextPtr ctxt;
+    PyObject *pyobj_ctxt;
+    xmlDocPtr RVT;
+    PyObject *pyobj_RVT;
+
+    if (!PyArg_ParseTuple(args, (char *)"OO:xsltRegisterPersistRVT", &pyobj_ctxt, &pyobj_RVT))
+        return(NULL);
+    ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
+    RVT = (xmlDocPtr) PyxmlNode_Get(pyobj_RVT);
+
+    c_retval = xsltRegisterPersistRVT(ctxt, RVT);
+    py_retval = libxml_intWrap((int) c_retval);
+    return(py_retval);
+}
+
 PyObject *
 libxslt_xsltResolveStylesheetAttributeSet(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
     xsltStylesheetPtr style;
@@ -1613,6 +1762,22 @@ libxslt_xsltRegisterAllExtras(ATTRIBUTE_UNUSED PyObject *self,ATTRIBUTE_UNUSED
     return(Py_None);
 }
 
+PyObject *
+libxslt_xsltTransformGetStyle(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
+    PyObject *py_retval;
+    xsltStylesheetPtr c_retval;
+    xsltTransformContextPtr ctxt;
+    PyObject *pyobj_ctxt;
+
+    if (!PyArg_ParseTuple(args, (char *)"O:xsltTransformGetStyle", &pyobj_ctxt))
+        return(NULL);
+    ctxt = (xsltTransformContextPtr) PytransformCtxt_Get(pyobj_ctxt);
+
+    c_retval = ctxt->style;
+    py_retval = libxslt_xsltStylesheetPtrWrap((xsltStylesheetPtr) c_retval);
+    return(py_retval);
+}
+
 PyObject *
 libxslt_xsltGetXIncludeDefault(ATTRIBUTE_UNUSED PyObject *self,ATTRIBUTE_UNUSED  PyObject *args) {
     PyObject *py_retval;
@@ -1669,21 +1834,6 @@ libxslt_xsltStylesheetGetMethodURI(ATTRIBUTE_UNUSED PyObject *self, PyObject *ar
     return(py_retval);
 }
 
-PyObject *
-libxslt_xsltDocumentFunction(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
-    xmlXPathParserContextPtr ctxt;
-    PyObject *pyobj_ctxt;
-    int nargs;
-
-    if (!PyArg_ParseTuple(args, (char *)"Oi:xsltDocumentFunction", &pyobj_ctxt, &nargs))
-        return(NULL);
-    ctxt = (xmlXPathParserContextPtr) PyxmlXPathParserContext_Get(pyobj_ctxt);
-
-    xsltDocumentFunction(ctxt, nargs);
-    Py_INCREF(Py_None);
-    return(Py_None);
-}
-
 PyObject *
 libxslt_xsltParseStylesheetDoc(ATTRIBUTE_UNUSED PyObject *self, PyObject *args) {
     PyObject *py_retval;
